package main

import (
	"encoding/base64"
	"encoding/json"
	"fmt"
	"github.com/urfave/cli/v2"
	"os"
	"path"
)

type KeyStore struct {
	PublicKey  string `json:"public_key"`
	PrivateKey string `json:"private_key"`
}

func LoadKey(c *cli.Context) (pub []byte, prv []byte, rerr error) {
	cp := GetKeyStorePath(c)
	keyName := c.String("key")
	if keyName == "" {
		return nil, nil, fmt.Errorf("flag 'key' should not be empty")
	}
	keyFile := path.Join(cp, fmt.Sprintf("%s.key.json", keyName))
	jbin, err := os.ReadFile(keyFile)
	if err != nil {
		return nil, nil, fmt.Errorf("failed to read key file: %w", err)
	}
	var jdat KeyStore
	err = json.Unmarshal(jbin, &jdat)
	if err != nil {
		return nil, nil, fmt.Errorf("failed to unmarshal key file: %w", err)
	}
	pub, err = base64.URLEncoding.DecodeString(jdat.PublicKey)
	if err != nil {
		return nil, nil, fmt.Errorf("failed to decode public key: %w", err)
	}
	prv, err = base64.URLEncoding.DecodeString(jdat.PrivateKey)
	if err != nil {
		return nil, nil, fmt.Errorf("failed to decode private key: %w", err)
	}
	return pub, prv, nil
}
